"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const SwaggerParser = require("swagger-parser");
const open_api_diff_error_impl_1 = require("../../common/open-api-diff-error-impl");
const parse_x_properties_1 = require("./common/parse-x-properties");
const path_builder_1 = require("./common/path-builder");
const typeCheckedOpenApi3Methods = {
    delete: undefined,
    get: undefined,
    head: undefined,
    options: undefined,
    patch: undefined,
    post: undefined,
    put: undefined,
    trace: undefined
};
const isOpenApi3Method = (propertyName) => Object.keys(typeCheckedOpenApi3Methods).indexOf(propertyName) >= 0;
const isRequestBodyObject = (requestBody) => !!requestBody && !!requestBody.content;
const parsedRequestBodyJsonSchema = (requestBodyObject, pathBuilder) => {
    if (isRequestBodyObject(requestBodyObject) && requestBodyObject.content['application/json']) {
        return {
            originalPath: pathBuilder.withChild('content').withChild('application/json').withChild('schema').build(),
            value: requestBodyObject.content['application/json'].schema
        };
    }
    return undefined;
};
const parsedRequestBody = (requestBody, pathBuilder) => {
    const originalPath = pathBuilder.withChild('requestBody');
    return {
        jsonSchema: parsedRequestBodyJsonSchema(requestBody, originalPath),
        originalValue: {
            originalPath: originalPath.build(),
            value: requestBody
        }
    };
};
const parseOperations = (pathItemObject, pathBuilder) => {
    return Object.keys(pathItemObject)
        .filter(isOpenApi3Method)
        .reduce((accumulator, method) => {
        const operationObject = pathItemObject[method];
        const originalPath = pathBuilder.withChild(method);
        const requestBody = operationObject
            ? parsedRequestBody(operationObject.requestBody, originalPath)
            : parsedRequestBody(undefined, originalPath);
        accumulator[method] = {
            originalValue: {
                originalPath: originalPath.build(),
                value: operationObject
            },
            requestBody
        };
        return accumulator;
    }, {});
};
const parsePaths = (paths, pathBuilder) => Object.keys(paths).reduce((accumulator, pathName) => {
    const pathItemObject = paths[pathName];
    const originalPath = pathBuilder.withChild(pathName);
    accumulator[pathName] = {
        operations: parseOperations(pathItemObject, originalPath),
        originalValue: {
            originalPath: originalPath.build(),
            value: pathItemObject
        },
        pathName
    };
    return accumulator;
}, {});
const validateOpenApi3 = (openApi3Spec, location) => __awaiter(this, void 0, void 0, function* () {
    try {
        const options = {
            dereference: { circular: false }
        };
        return yield SwaggerParser.validate(openApi3Spec, options);
    }
    catch (error) {
        throw new open_api_diff_error_impl_1.OpenApiDiffErrorImpl('OPENAPI_DIFF_VALIDATE_OPENAPI_3_ERROR', `Validation errors in ${location}`, error);
    }
});
const parseOpenApi3Spec = (spec) => {
    const pathBuilder = path_builder_1.PathBuilder.createRootPathBuilder();
    return {
        format: 'openapi3',
        paths: parsePaths(spec.paths, pathBuilder.withChild('paths')),
        xProperties: parse_x_properties_1.parseXPropertiesInObject(spec, pathBuilder)
    };
};
exports.validateAndParseOpenApi3Spec = (openApi3Spec, location) => __awaiter(this, void 0, void 0, function* () {
    const validatedOpenApi3Spec = yield validateOpenApi3(openApi3Spec, location);
    return parseOpenApi3Spec(validatedOpenApi3Spec);
});
